@@ -56,18 +56,18 @@ class _UtilityHandler(object):
     def __init__(self):
         self._utilities = {}
 
-    def provide(self, iface, obj):
-        global _interfaces
+    # FIXME: How to replace a utility properly
+    def provide(self, iface, obj, replace=False):
         if not issubclass(iface, Interface):
             raise TypeError(
                 "iface must be an Interface subclass and not %r" % iface)
 
-        if iface in self._utilities:
-            raise AlreadyImplementedError("%s is already implemented" % iface)
+        if not replace:
+            if iface in self._utilities:
+                raise AlreadyImplementedError("%s is already implemented" % iface)
         self._utilities[iface] = obj
 
     def get(self, iface):
-        global _interfaces
         if not issubclass(iface, Interface):
             raise TypeError(
                 "iface must be an Interface subclass and not %r" % iface)
@@ -77,7 +77,7 @@ class _UtilityHandler(object):
 
         return self._utilities[iface]
 
-def provide_utility(iface, utility):
+def provide_utility(iface, utility, replace=False):
     """
     Set the utility for the named interface. If the utility is already
     set, an {AlreadyImplementedError} is raised.
@@ -85,8 +85,7 @@ def provide_utility(iface, utility):
     @param iface: interface to set the utility for.
     @param utility: utility providing the interface.
     """
-    global _handler
-    _handler.provide(iface, utility)
+    _handler.provide(iface, utility, replace)
 
 def get_utility(iface):
     """
@@ -97,7 +96,6 @@ def get_utility(iface):
     @type iface: utility providing the interface
     """
 
-    global _handler
     return _handler.get(iface)
 
 _handler = _UtilityHandler()
