@@ -34,12 +34,13 @@ import gtk
 from gtk import gdk
 
 from kiwi.accessor import kgetattr
-from kiwi.datatypes import converter, currency, number
+from kiwi.datatypes import converter, number, Decimal
+from kiwi.currency import currency # after datatypes
 from kiwi.log import Logger
 from kiwi.python import slicerange
 from kiwi.utils import PropertyObject, gsignal, gproperty, type_register
 
-_ = gettext.gettext
+_ = lambda m: gettext.dgettext('kiwi', m)
 
 log = Logger('objectlist')
 
@@ -73,8 +74,9 @@ class Column(PropertyObject, gobject.GObject):
     gproperty('radio', bool, default=False)
     gproperty('cache', bool, default=False)
     gproperty('use-stock', bool, default=False)
+    gproperty('use-markup', bool, default=False)
     gproperty('icon-size', gtk.IconSize, default=gtk.ICON_SIZE_MENU)
-    gproperty('editable_attribute', str)
+    gproperty('editable-attribute', str)
     #gproperty('title_pixmap', str)
 
     # This can be set in subclasses, to be able to allow custom
@@ -144,6 +146,7 @@ class Column(PropertyObject, gobject.GObject):
             specified.
         @keyword editable_attribute: a string which is the attribute
             which should decide if the cell is editable or not.
+        @keyword use_markup: If true, the text will be rendered with markup
         @keyword title_pixmap: (TODO) if set to a filename a pixmap will be
             used *instead* of the title set. The title string will still be
             used to identify the column in the column selection and in a
@@ -209,21 +212,12 @@ class Column(PropertyObject, gobject.GObject):
         del namespace['attribute']
         return "<%s %s: %s>" % (self.__class__.__name__, attr, namespace)
 
-    # XXX: Replace these two with a gazpacho loader adapter
-    def __str__(self):
-        if self.data_type is None:
-            data_type = ''
-        else:
-            data_type = self.data_type.__name__
-
-        return "%s|%s|%s|%s|%d|%s|%s|%d|%s|%d" % \
-               (self.attribute, self.title, data_type, self.visible,
-                self.justify, self.tooltip, self.format, self.width,
-                self.sorted, self.order)
-
     def as_string(self, data):
         data_type = self.data_type
         if (self.format or
+            data_type == float or
+            data_type == Decimal or
+            data_type == currency or
             data_type == datetime.date or
             data_type == datetime.datetime or
             data_type == datetime.time):
@@ -236,38 +230,6 @@ class Column(PropertyObject, gobject.GObject):
 
         return text
 
-    def from_string(cls, data_string):
-        fields = data_string.split('|')
-        if len(fields) != 10:
-            msg = 'every column should have 10 fields, not %d' % len(fields)
-            raise ValueError(msg)
-
-        # the attribute is mandatory
-        if not fields[0]:
-            raise TypeError
-
-        column = cls(fields[0])
-        column.title = fields[1] or ''
-        column.data_type = str2type(fields[2])
-        column.visible = str2bool(fields[3])
-        column.justify = str2enum(fields[4], gtk.JUSTIFY_LEFT)
-        column.tooltip = fields[5]
-        column.format = fields[6]
-
-        try:
-            column.width = int(fields[7])
-        except ValueError:
-            pass
-
-        column.sorted = str2bool(fields[8])
-        column.order = str2enum(fields[9], gtk.SORT_ASCENDING) \
-                     or gtk.SORT_ASCENDING
-
-        # XXX: expand, remember to sync with __str__
-
-        return column
-    from_string = classmethod(from_string)
-
 class SequentialColumn(Column):
     """I am a column which will display a sequence of numbers, which
     represent the row number. The value is independent of the data in
@@ -472,20 +434,20 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
     # emitted when empty or non-empty status changes
     gsignal('has-rows', bool)
 
-    # this property is used to serialize the columns of a ObjectList. The format
-    # is a big string with '^' as the column separator and '|' as the field
-    # separator
-    gproperty('column-definitions', str, nick="ColumnDefinitions")
     gproperty('selection-mode', gtk.SelectionMode,
               default=gtk.SELECTION_BROWSE, nick="SelectionMode")
 
     def __init__(self, columns=[],
-                 instance_list=None,
-                 mode=gtk.SELECTION_BROWSE):
+                 objects=None,
+                 mode=gtk.SELECTION_BROWSE,
+                 sortable=False,
+                 model=None):
         """
         @param columns:       a list of L{Column}s
-        @param instance_list: a list of objects to be inserted or None
+        @param objects:       a list of objects to be inserted or None
         @param mode:          selection mode
+        @param sortable:      whether the user can sort the list
+        @param model:         gtk.TreeModel to use or None to create one
         """
         # allow to specify only one column
         if isinstance(columns, Column):
@@ -500,16 +462,14 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         #elif mode == gtk.SELECTION_EXTENDED:
         #    raise TypeError("gtk.SELECTION_EXTENDED is deprecated")
 
+        self._sortable = sortable
+
+        self._columns = []
         # Mapping of instance id -> treeiter
         self._iters = {}
         self._cell_data_caches = {}
-        self._columns_configured = False
         self._autosize = True
         self._vscrollbar = None
-        # by default we are unordered. This index points to the column
-        # definition of the column that dictates the order, in case there is
-        # any
-        self._sort_column_index = -1
 
         gtk.ScrolledWindow.__init__(self)
 
@@ -519,7 +479,9 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
         self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
 
-        self._model = gtk.ListStore(object)
+        if not model:
+            model = gtk.ListStore(object)
+        self._model = model
         self._model.connect('row-inserted', self._on_model__row_inserted)
         self._model.connect('row-deleted', self._on_model__row_deleted)
         self._treeview = gtk.TreeView(self._model)
@@ -540,22 +502,15 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         # when setting the column definition the columns are created
         self.set_columns(columns)
 
-        if instance_list:
-            self._treeview.freeze_notify()
-            self._load(instance_list, clear=True)
-            self._treeview.thaw_notify()
-
-        if self._sort_column_index != -1:
-            column = self._columns[self._sort_column_index]
-            self._model.set_sort_column_id(self._sort_column_index,
-                                           column.order)
+        if objects:
+            self.add_list(objects, clear=True)
 
         # Set selection mode last to avoid spurious events
         selection = self._treeview.get_selection()
         selection.connect("changed", self._on_selection__changed)
 
         # Select the first item if no items are selected
-        if mode != gtk.SELECTION_NONE and instance_list:
+        if mode != gtk.SELECTION_NONE and objects:
             selection.select_iter(self._model[COL_MODEL].iter)
 
         # Depends on treeview and selection being set up
@@ -693,27 +648,14 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     # Properties
 
-    def prop_set_column_definition(self, value):
-        self.set_columns(value)
-        return value
-
     def prop_set_selection_mode(self, mode):
         self.set_selection_mode(mode)
 
     def prop_get_selection_mode(self):
         return self.get_selection_mode()
 
-    # Model
-
-    def _on_model__row_inserted(self, model, path, iter):
-        if len(model) == 1:
-            self.emit('has-rows', True)
-
-    def _on_model__row_deleted(self, model, path):
-        if not len(model):
-            self.emit('has-rows', False)
-
     # Columns handling
+
     def _load(self, instances, clear):
         # do nothing if empty list or None provided
         model = self._model
@@ -783,14 +725,11 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
             self._treeview.columns_autosize()
             self._autosize = False
 
-    def _setup_columns(self):
-        if self._columns_configured:
-            return
-
+    def _setup_columns(self, columns):
         searchable = None
         sorted = None
         expand = False
-        for column in self._columns:
+        for column in columns:
             if column.searchable:
                 if searchable:
                     raise ValueError("Can't make column %s searchable, column"
@@ -806,26 +745,31 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
             if column.expand:
                 expand = True
 
-        for column in self._columns:
+        self._sortable = self._sortable or sorted
+
+        for column in columns:
             self._setup_column(column)
 
         if not expand:
             column = gtk.TreeViewColumn()
             self._treeview.append_column(column)
 
-        self._columns_configured = True
-
     def _setup_column(self, column):
         # You can't subclass bool, so this is okay
         if (column.data_type is bool and column.format):
             raise TypeError("format is not supported for boolean columns")
 
         index = self._columns.index(column)
-        self._model.set_sort_func(index, self._sort_function)
         treeview_column = self._treeview.get_column(index)
         if treeview_column is None:
             treeview_column = self._create_column(column)
 
+        if self._sortable:
+            self._model.set_sort_func(index,
+                                      self._model_sort_func,
+                                      (column, column.attribute))
+            treeview_column.set_sort_column_id(index)
+
         renderer, renderer_prop = self._guess_renderer_for_type(column)
         if column.on_attach_renderer:
             column.on_attach_renderer(renderer)
@@ -857,7 +801,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
                                            (column, renderer_prop))
         treeview_column.set_visible(column.visible)
 
-        treeview_column.connect("clicked", self._on_column__clicked, column)
         if column.width:
             treeview_column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             treeview_column.set_fixed_width(column.width)
@@ -871,7 +814,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
             treeview_column.set_expand(True)
 
         if column.sorted:
-            self._sort_column_index = index
             treeview_column.set_sort_indicator(True)
 
         if column.width:
@@ -882,7 +824,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
                 raise TypeError("Unsupported data type for "
                                 "searchable column: %s" % column.data_type)
             self._treeview.set_search_column(index)
-            self._treeview.set_search_equal_func(self._search_equal_func,
+            self._treeview.set_search_equal_func(self._treeview_search_equal_func,
                                                  column)
 
         if column.radio:
@@ -912,9 +854,197 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         # setup the button to show the popup menu
         button = self._get_column_button(treeview_column)
         button.connect('button-release-event',
-                       self._on_header__button_release_event)
+                       self._on_treeview_header__button_release_event)
         return treeview_column
 
+    def _guess_renderer_for_type(self, column):
+        """Gusses which CellRenderer we should use for a given type.
+        It also set the property of the renderer that depends on the model,
+        in the renderer.
+        """
+
+        # TODO: Move to column
+        data_type = column.data_type
+        if data_type is bool:
+            renderer = gtk.CellRendererToggle()
+            if column.editable:
+                renderer.set_property('activatable', True)
+                # Boolean can be either a radio or a checkbox.
+                # Changes are handled by the toggled callback, which
+                # should only be connected if the column is editable.
+                if column.radio:
+                    renderer.set_radio(True)
+                    cb = self._on_renderer_toggle_radio__toggled
+                else:
+                    cb = self._on_renderer_toggle_check__toggled
+                renderer.connect('toggled', cb, self._model, column.attribute)
+            prop = 'active'
+        elif column.use_stock or data_type == gdk.Pixbuf:
+            renderer = gtk.CellRendererPixbuf()
+            prop = 'pixbuf'
+            if column.editable:
+                raise TypeError("use-stock columns cannot be editable")
+        elif issubclass(data_type, (datetime.date, datetime.time,
+                                    basestring, number,
+                                    currency)):
+            renderer = gtk.CellRendererText()
+            if column.use_markup:
+                prop = 'markup'
+            else:
+                prop = 'text'
+            if column.editable:
+                renderer.set_property('editable', True)
+                renderer.connect('edited', self._on_renderer_text__edited,
+                                 self._model, column.attribute, column,
+                                 column.from_string)
+
+        else:
+            raise ValueError("the type %s is not supported yet" % data_type)
+
+        return renderer, prop
+
+    # selection methods
+    def _select_and_focus_row(self, row_iter):
+        self._treeview.set_cursor(self._model[row_iter].path)
+
+    # handlers & callbacks
+
+    # Model
+    def _on_model__row_inserted(self, model, path, iter):
+        if len(model) == 1:
+            self.emit('has-rows', True)
+
+    def _on_model__row_deleted(self, model, path):
+        if not len(model):
+            self.emit('has-rows', False)
+
+    def _model_sort_func(self, model, iter1, iter2, (column, attr)):
+        "This method is used to sort the GtkTreeModel"
+        return column.compare(
+            column.get_attribute(model[iter1][COL_MODEL], attr),
+            column.get_attribute(model[iter2][COL_MODEL], attr))
+
+    # Selection
+    def _on_selection__changed(self, selection):
+        "This method is used to proxy selection::changed to selection-changed"
+        mode = selection.get_mode()
+        if mode == gtk.SELECTION_MULTIPLE:
+            item = self.get_selected_rows()
+        elif mode in (gtk.SELECTION_SINGLE, gtk.SELECTION_BROWSE):
+            item = self.get_selected()
+        else:
+            raise AssertionError
+        self.emit('selection-changed', item)
+
+    # ScrolledWindow
+    def _on_scrolled_window__realize(self, widget):
+        toplevel = widget.get_toplevel()
+        self._popup_window.set_transient_for(toplevel)
+        self._popup_window.set_destroy_with_parent(True)
+
+    def _on_scrolled_window__size_allocate(self, widget, allocation):
+        """Resize the Vertical Scrollbar to make it smaller and let space
+        for the popup button. Also put that button there.
+        """
+        old_alloc = self._vscrollbar.get_allocation()
+        height = self._get_header_height()
+        new_alloc = gtk.gdk.Rectangle(old_alloc.x, old_alloc.y + height,
+                                      old_alloc.width,
+                                      old_alloc.height - height)
+        self._vscrollbar.size_allocate(new_alloc)
+        # put the popup_window in its position
+        gdk_window = self.window
+        if gdk_window:
+            winx, winy = gdk_window.get_origin()
+            self._popup_window.move(winx + old_alloc.x,
+                                    winy + old_alloc.y)
+
+    # TreeView
+    def _treeview_search_equal_func(self, model, tree_column, key, treeiter, column):
+        "for searching inside the treeview"
+        data = column.get_attribute(model[treeiter][COL_MODEL],
+                                    column.attribute, None)
+        if data.startswith(key):
+            return False
+        return True
+
+    def _on_treeview_header__button_release_event(self, button, event):
+        if event.button == 3:
+            self._popup.popup(event)
+
+        return False
+
+    def _after_treeview__row_activated(self, treeview, path, view_column):
+        "After activated (double clicked or pressed enter) on a row"
+        try:
+            row = self._model[path]
+        except IndexError:
+            print 'path %s was not found in model: %s' % (
+                path, map(list, self._model))
+            return
+        item = row[COL_MODEL]
+        self.emit('row-activated', item)
+
+    def _on_treeview__button_press_event(self, treeview, event):
+        "Generic button-press-event handler to be able to catch double clicks"
+        if event.type == gtk.gdk._2BUTTON_PRESS:
+            selection = self._treeview.get_selection()
+            mode = selection.get_mode()
+            if mode == gtk.SELECTION_MULTIPLE:
+                item = self.get_selected_rows()
+            else:
+                item = self.get_selected()
+            self.emit('double-click', item)
+
+    # CellRenderers
+    def _cell_data_text_func(self, tree_column, renderer, model, treeiter,
+                             (column, renderer_prop)):
+        "To render the data of a cell renderer text"
+        row = model[treeiter]
+        if column.editable_attribute:
+            data = column.get_attribute(row[COL_MODEL],
+                                        column.editable_attribute, None)
+            data_type = column.data_type
+            if isinstance(renderer, gtk.CellRendererToggle):
+                renderer.set_property('activatable', data)
+            elif isinstance(renderer, gtk.CellRendererText):
+                renderer.set_property('editable', data)
+            else:
+                raise AssertionError
+
+        if column.cache:
+            cache = self._cell_data_caches[column.attribute]
+            path = row.path[0]
+            if path in cache:
+                data = cache[path]
+            else:
+                data = column.get_attribute(row[COL_MODEL],
+                                            column.attribute, None)
+                cache[path] = data
+        else:
+            data = column.get_attribute(row[COL_MODEL],
+                                        column.attribute, None)
+
+        text = column.as_string(data)
+
+        renderer.set_property(renderer_prop, text)
+
+        if column.renderer_func:
+            column.renderer_func(renderer, data)
+
+    def _cell_data_pixbuf_func(self, tree_column, renderer, model, treeiter,
+                               (column, renderer_prop)):
+        "To render the data of a cell renderer pixbuf"
+        row = model[treeiter]
+        data = column.get_attribute(row[COL_MODEL],
+                                    column.attribute, None)
+        pixbuf = self.render_icon(data, column.icon_size)
+        renderer.set_property(renderer_prop, pixbuf)
+
+    def _on_renderer__toggled(self, renderer, path, column):
+        setattr(self._model[path][COL_MODEL], column.attribute,
+                not renderer.get_active())
+
     def _on_renderer_toggle_check__toggled(self, renderer, path, model, attr):
         obj = model[path][COL_MODEL]
         value = not getattr(obj, attr, None)
@@ -957,106 +1087,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         setattr(obj, attr, value)
         self.emit('cell-edited', obj, attr)
 
-    def _guess_renderer_for_type(self, column):
-        """Gusses which CellRenderer we should use for a given type.
-        It also set the property of the renderer that depends on the model,
-        in the renderer.
-        """
-
-        # TODO: Move to column
-        data_type = column.data_type
-        if data_type is bool:
-            renderer = gtk.CellRendererToggle()
-            if column.editable:
-                renderer.set_property('activatable', True)
-                # Boolean can be either a radio or a checkbox.
-                # Changes are handled by the toggled callback, which
-                # should only be connected if the column is editable.
-                if column.radio:
-                    renderer.set_radio(True)
-                    cb = self._on_renderer_toggle_radio__toggled
-                else:
-                    cb = self._on_renderer_toggle_check__toggled
-                renderer.connect('toggled', cb, self._model, column.attribute)
-            prop = 'active'
-        elif column.use_stock or data_type == gdk.Pixbuf:
-            renderer = gtk.CellRendererPixbuf()
-            prop = 'pixbuf'
-            if column.editable:
-                raise TypeError("use-stock columns cannot be editable")
-        elif issubclass(data_type, (datetime.date, datetime.time,
-                                    basestring, number,
-                                    currency)):
-            renderer = gtk.CellRendererText()
-            prop = 'text'
-            if column.editable:
-                renderer.set_property('editable', True)
-                renderer.connect('edited', self._on_renderer_text__edited,
-                                 self._model, column.attribute, column,
-                                 column.from_string)
-
-        else:
-            raise ValueError("the type %s is not supported yet" % data_type)
-
-        return renderer, prop
-
-    def _search_equal_func(self, model, tree_column, key, treeiter, column):
-        data = column.get_attribute(model[treeiter][COL_MODEL],
-                                    column.attribute, None)
-        if data.startswith(key):
-            return False
-        return True
-
-    def _cell_data_text_func(self, tree_column, renderer, model, treeiter,
-                             (column, renderer_prop)):
-
-        row = model[treeiter]
-        if column.editable_attribute:
-            data = column.get_attribute(row[COL_MODEL],
-                                        column.editable_attribute, None)
-            data_type = column.data_type
-            if isinstance(renderer, gtk.CellRendererToggle):
-                renderer.set_property('activatable', data)
-            elif isinstance(renderer, gtk.CellRendererText):
-                renderer.set_property('editable', data)
-            else:
-                raise AssertionError
-
-        if column.cache:
-            cache = self._cell_data_caches[column.attribute]
-            path = row.path[0]
-            if path in cache:
-                data = cache[path]
-            else:
-                data = column.get_attribute(row[COL_MODEL],
-                                            column.attribute, None)
-                cache[path] = data
-        else:
-            data = column.get_attribute(row[COL_MODEL],
-                                        column.attribute, None)
-
-        text = column.as_string(data)
-
-        renderer.set_property(renderer_prop, text)
-
-        if column.renderer_func:
-            column.renderer_func(renderer, data)
-
-    def _cell_data_pixbuf_func(self, tree_column, renderer, model, treeiter,
-                               (column, renderer_prop)):
-        row = model[treeiter]
-        data = column.get_attribute(row[COL_MODEL],
-                                    column.attribute, None)
-        pixbuf = self.render_icon(data, column.icon_size)
-        renderer.set_property(renderer_prop, pixbuf)
-
-    def _on_header__button_release_event(self, button, event):
-        if event.button == 3:
-            self._popup.popup(event)
-            return False
-
-        return False
-
     def _on_renderer__edited(self, renderer, path, value, column):
         data_type = column.data_type
         if data_type in number:
@@ -1065,90 +1095,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         # XXX convert new_text to the proper data type
         setattr(self._model[path][COL_MODEL], column.attribute, value)
 
-    def _on_renderer__toggled(self, renderer, path, column):
-        setattr(self._model[path][COL_MODEL], column.attribute,
-                not renderer.get_active())
-
-    def _clear_columns(self):
-        while self._treeview.get_columns():
-            self._treeview.remove_column(self._treeview.get_column(COL_MODEL))
-
-        self._popup.clean()
-
-        self._columns_configured = False
-
-    # selection methods
-    def _select_and_focus_row(self, row_iter):
-        self._treeview.set_cursor(self._model[row_iter].path)
-
-    def _sort_function(self, model, iter1, iter2):
-        column = self._columns[self._sort_column_index]
-        attr = column.attribute
-        return column.compare(
-            column.get_attribute(model[iter1][COL_MODEL], attr),
-            column.get_attribute(model[iter2][COL_MODEL], attr))
-
-    def _on_column__clicked(self, treeview_column, column):
-        # this means we are not sorting at all
-        if self._sort_column_index == -1:
-            return
-
-        old_treeview_column = self._treeview.get_column(
-            self._sort_column_index)
-
-        if treeview_column is old_treeview_column:
-            # same column, so reverse the order
-            if column.order == gtk.SORT_ASCENDING:
-                new_order = gtk.SORT_DESCENDING
-            elif column.order == gtk.SORT_DESCENDING:
-                new_order = gtk.SORT_ASCENDING
-            else:
-                raise AssertionError
-        else:
-            # new column, sort ascending
-            new_order = gtk.SORT_ASCENDING
-            self._sort_column_index = self._columns.index(column)
-            # cosmetic changes
-            old_treeview_column.set_sort_indicator(False)
-            treeview_column.set_sort_indicator(True)
-
-        column.order = new_order
-        treeview_column.set_sort_order(new_order)
-
-        # This performs the actual ordering
-        self._model.set_sort_column_id(self._sort_column_index, new_order)
-
-    # handlers
-    def _after_treeview__row_activated(self, treeview, path, view_column):
-        try:
-            row = self._model[path]
-        except IndexError:
-            print 'path %s was not found in model: %s' % (
-                path, map(list, self._model))
-            return
-        item = row[COL_MODEL]
-        self.emit('row-activated', item)
-
-    def _on_selection__changed(self, selection):
-        mode = selection.get_mode()
-        if mode == gtk.SELECTION_MULTIPLE:
-            item = self.get_selected_rows()
-        elif mode in (gtk.SELECTION_SINGLE, gtk.SELECTION_BROWSE):
-            item = self.get_selected()
-        else:
-            raise AssertionError
-        self.emit('selection-changed', item)
-
-    def _on_treeview__button_press_event(self, treeview, event):
-        if event.type == gtk.gdk._2BUTTON_PRESS:
-            selection = self._treeview.get_selection()
-            mode = selection.get_mode()
-            if mode == gtk.SELECTION_MULTIPLE:
-                item = self.get_selected_rows()
-            else:
-                item = self.get_selected()
-            self.emit('double-click', item)
-
     # hacks
     def _get_column_button(self, column):
         """Return the button widget of a particular TreeViewColumn.
@@ -1197,28 +1143,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         alloc = button.get_allocation()
         return alloc.height
 
-    def _on_scrolled_window__realize(self, widget):
-        toplevel = widget.get_toplevel()
-        self._popup_window.set_transient_for(toplevel)
-        self._popup_window.set_destroy_with_parent(True)
-
-    def _on_scrolled_window__size_allocate(self, widget, allocation):
-        """Resize the Vertical Scrollbar to make it smaller and let space
-        for the popup button. Also put that button there.
-        """
-        old_alloc = self._vscrollbar.get_allocation()
-        height = self._get_header_height()
-        new_alloc = gtk.gdk.Rectangle(old_alloc.x, old_alloc.y + height,
-                                      old_alloc.width,
-                                      old_alloc.height - height)
-        self._vscrollbar.size_allocate(new_alloc)
-        # put the popup_window in its position
-        gdk_window = self.window
-        if gdk_window:
-            winx, winy = gdk_window.get_origin()
-            self._popup_window.move(winx + old_alloc.x,
-                                    winy + old_alloc.y)
-
     # end of the popup button hack
 
     #
@@ -1257,29 +1181,32 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         tree_columns = self._treeview.get_columns()
         return tree_columns[index]
 
-    def set_columns(self, value):
-        """This function can be called in two different ways:
-         - value is a string with the column definitions in a special format
-           (see column-definitions property at the beginning of this class)
+    def _clear_columns(self):
+        # Reset the sort function for all model columns
+        model = self._model
+        for i, column in enumerate(self._columns):
+            # Bug in PyGTK, it should be possible to remove a sort func.
+            model.set_sort_func(i, lambda m, i1, i2: -1)
 
-         - value is a list/tuple of Column objects
+        # Remove all columns
+        treeview = self._treeview
+        while treeview.get_columns():
+            treeview.remove_column(treeview.get_column(0))
+
+        self._popup.clean()
+        self._columns = []
+
+    def set_columns(self, columns):
+        """
+        @param columns: a sequence of L{Column} objects.
         """
 
-        if isinstance(value, basestring):
-            self._columns_string = value
-            self._columns = []
-            for col in value.split('^'):
-                if not col:
-                    continue
-                self._columns.append(Column.from_string(col))
-        elif isinstance(value, (list, tuple)):
-            self._columns = value
-            self._columns_string = '^'.join([str(col) for col in value])
-        else:
-            raise ValueError("value should be a string of a list of columns")
+        if not isinstance(columns, (list, tuple)):
+            raise ValueError("columns must be a list or a tuple")
 
         self._clear_columns()
-        self._setup_columns()
+        self._columns = columns
+        self._setup_columns(columns)
 
     def append(self, instance, select=False):
         """Adds an instance to the list.
@@ -1353,15 +1280,17 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         self._clear_cache_for_iter(treeiter)
         self._model.row_changed(self._model[treeiter].path, treeiter)
 
-    def refresh(self):
+    def refresh(self, view_only=False):
         """
         Reloads the values from all objects.
+
+        @param view_only: if True, only force a refresh of the
+            visible part of this objectlist's Treeview.
         """
-        # XXX: Optimize this to only reload items, no need to remove/readd
-        model = self._model
-        instances = [row[COL_MODEL] for row in model]
-        model.clear()
-        self.add_list(instances)
+        if view_only:
+            self._treeview.queue_draw()
+        else:
+            self._model.foreach(gtk.TreeModel.row_changed)
 
     def set_column_visibility(self, column_index, visibility):
         treeview_column = self._treeview.get_column(column_index)
@@ -1564,6 +1493,63 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
 type_register(ObjectList)
 
+class ObjectRow(object):
+    def __init__(self, iter):
+        self.iter = iter
+
+class ObjectTree(ObjectList):
+    def __init__(self, columns=[], objects=None, mode=gtk.SELECTION_BROWSE,
+                 sortable=False, model=None):
+        if not model:
+            model = gtk.TreeStore(object)
+        ObjectList.__init__(self, columns, objects, mode, sortable, model)
+
+    def _append_internal(self, parent, instance, select, prepend):
+        if parent is not None and not isinstance(parent, ObjectRow):
+            raise TypeError("parent must be an ObjectRow or None")
+
+        # Freeze and save original selection mode to avoid blinking
+        self._treeview.freeze_notify()
+
+        if parent:
+            parent_iter = parent.iter
+        else:
+            parent_iter = None
+
+        if prepend:
+            row_iter = self._model.prepend(parent_iter, (instance,))
+        else:
+            row_iter = self._model.append(parent_iter, (instance,))
+
+        self._iters[id(instance)] = row_iter
+
+        if self._autosize:
+            self._treeview.columns_autosize()
+
+        if select:
+            self._select_and_focus_row(row_iter)
+        self._treeview.thaw_notify()
+
+        return ObjectRow(row_iter)
+
+    def append(self, parent, instance, select=False):
+        """
+        @param parent: ObjectRow of the parent
+        @param instance: the instance to be added
+        @param select: select the row
+        """
+        return self._append_internal(parent, instance, select, prepend=False)
+
+    def prepend(self, parent, instance, select=False):
+        """
+        @param parent: ObjectRow of the parent
+        @param instance: the instance to be added
+        @param select: select the row
+        """
+        return self._append_internal(parent, instance, select, prepend=True)
+
+type_register(ObjectTree)
+
 class ListLabel(gtk.HBox):
     """I am a subclass of a GtkHBox which you can use if you want
     to vertically align a label with a column
