@@ -1,23 +1,23 @@
 #
 # Kiwi: a Framework and Enhanced Widgets for Python
 #
-# Copyright (C) 2001-2005 Async Open Source
+# Copyright (C) 2001-2006 Async Open Source
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2.1 of the License, or (at your option) any later version.
-# 
+#
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 # USA
-# 
+#
 # Author(s): Christian Reis <kiko@async.com.br>
 #            Jon Nelson <jnelson@securepipe.com>
 #            Lorenzo Gil Sanchez <lgs@sicem.biz>
@@ -30,32 +30,24 @@ Defines the View classes that are included in the Kiwi Framework, which
 are the base of Delegates and Proxies.
 """
 
+import re
 import string
 
 import gobject
 import gtk
+from gtk import gdk
 
-from kiwi import _warn
-from kiwi.environ import require_gazpacho_loader
-from kiwi.interfaces import MixinSupportValidation
-from kiwi.proxies import Proxy
-from kiwi.utils import gsignal
+from kiwi.environ import is_gazpacho_required
+from kiwi.interfaces import IValidatableProxyWidget
+from kiwi.log import Logger
+from kiwi.utils import gsignal, type_register
 from kiwi.ui.gadgets import quit_if_last
+from kiwi.ui.proxy import Proxy
 
-try:
-    from kiwi.ui.gazpacholoader import GazpachoWidgetTree as WidgetTree
-except ImportError, e:
-    if require_gazpacho_loader:
-        raise RuntimeError("Gazpacho is required, but could not be found: %s"
-                           % e)
-    else:
-        try:
-            from kiwi.ui.libgladeloader import LibgladeWidgetTree as WidgetTree
-        except ImportError:
-            raise RuntimeError("Could not find a glade parser library")
-        
-_non_interactive = [
-    gtk.Label, 
+log = Logger('view')
+
+_non_interactive = (
+    gtk.Label,
     gtk.Alignment,
     gtk.AccelLabel,
     gtk.Arrow,
@@ -75,15 +67,18 @@ _non_interactive = [
     gtk.VButtonBox,
     gtk.VPaned,
     gtk.VSeparator,
-    gtk.Window, 
-]
+    gtk.Window,
+)
 
-_non_interactive = tuple(_non_interactive)
+color_red = gdk.color_parse('red')
+color_black = gdk.color_parse('black')
 
 #
 # Signal brokers
 #
 
+method_regex = re.compile(r'^(on|after)_(\w+)__(\w+)$')
+
 class SignalBroker(object):
     def __init__(self, view, controller):
         methods = controller._get_all_methods()
@@ -92,13 +87,13 @@ class SignalBroker(object):
     def _do_connections(self, view, methods):
         """This method allows subclasses to add more connection mechanism"""
         self._autoconnect_by_method_name(view, methods)
-        
+
     def _autoconnect_by_method_name(self, view, methods):
         """
         Offers autoconnection of widget signals based on function names.
         You simply need to define your controller method in the format::
 
-            def on_widget_name__signal_name(self, widget, *args):
+            def on_widget_name__signal_name(self, widget):
 
         In other words, start the method by "on_", followed by the
         widget name, followed by two underscores ("__"), followed by the
@@ -107,39 +102,21 @@ class SignalBroker(object):
         signal name.
         """
         self._autoconnected = {}
+
         for fname in methods.keys():
             # `on_x__y' has 7 chars and is the smallest possible handler
             # (though illegal, of course, since the signal name x is bogus)
             if len(fname) < 7:
                 continue
-            if fname[:3] == "on_":
-                after = 0
-            elif fname[:6] == "after_":
-                after = 1
-            else:
-                continue
-            # cut out `on_' or `after'
-            f = string.split(fname, "_", 1)
-            if len(f) < 2:
-                continue
-            f = f[1]
-            # separate widget from signal; for instance: 
-            # "on_foo_bar__widget__clicked()" -> "foo_bar", "widget", "clicked"
-            f = string.split(f, "__")
-            if len(f) < 2:
-                continue
-            # signal is the last element
-            signal = f[-1]
-            # widget name is everything up to the last __
-            # "foo_bar", "widget" -> "foo_bar__widget"
-            w_name = string.join(f[:-1], "__")
-            if not w_name:
+            match = method_regex.match(fname)
+            if match is None:
                 continue
+            after, w_name, signal = match.groups()
             widget = getattr(view, w_name, None)
             if widget is None:
                 raise AttributeError("couldn't find widget %s in %s"
                                      % (w_name, view))
-            if not isinstance(widget, (gtk.Widget, gtk.Action)):
+            if not isinstance(widget, gobject.GObject):
                 raise AttributeError("%s (%s) is not a widget or an action "
                                      "and can't be connected to"
                                      % (w_name, widget))
@@ -147,22 +124,20 @@ class SignalBroker(object):
             # being called unbound and lacking, thus, "self".
             try:
                 if after:
-                   signal_id = widget.connect_after(signal, methods[fname])
+                    signal_id = widget.connect_after(signal, methods[fname])
                 else:
-                   signal_id = widget.connect(signal, methods[fname])
+                    signal_id = widget.connect(signal, methods[fname])
             except TypeError:
-                _warn("Widget %s doesn't provide a signal %s"
-                     % (widget.__class__, signal))
-                continue
-            if not self._autoconnected.has_key(widget):
-                self._autoconnected[widget] = []
-            self._autoconnected[widget].append((signal, signal_id))
+                raise TypeError("Widget %s doesn't provide a signal %s" % (
+                                widget.__class__, signal))
+            self._autoconnected.setdefault(widget, []).append((
+                signal, signal_id))
 
     def handler_block(self, widget, signal_name):
         signals = self._autoconnected
         if not widget in signals:
             return
-            
+
         for signal, signal_id in signals[widget]:
             if signal_name is None or signal == signal_name:
                 widget.handler_block(signal_id)
@@ -171,24 +146,21 @@ class SignalBroker(object):
         signals = self._autoconnected
         if not widget in signals:
             return
-        
+
         for signal, signal_id in signals[widget]:
             if signal_name is None or signal == signal_name:
                 widget.handler_unblock(signal_id)
 
     def disconnect_autoconnected(self):
         for widget, signals in self._autoconnected.items():
-            for unused, signal_id in signals:
-                widget.disconnect(signal_id)
-        
-class GladeSignalBroker(SignalBroker):
-    def __init__(self, view, controller):
-        SignalBroker.__init__(self, view, controller)
+            for signal in signals:
+                widget.disconnect(signal[1])
 
+class GladeSignalBroker(SignalBroker):
     def _do_connections(self, view, methods):
         super(GladeSignalBroker, self)._do_connections(view, methods)
         self._connect_glade_signals(view, methods)
-        
+
     def _connect_glade_signals(self, view, methods):
         # mainly because the two classes cannot have a common base
         # class. studying the class layout carefully or using
@@ -203,8 +175,8 @@ class GladeSignalBroker(SignalBroker):
         for name, method in methods.items():
             if callable(method):
                 dict[name] = method
-        view.glade_adaptor.signal_autoconnect(dict)
-        
+        view._glade_adaptor.signal_autoconnect(dict)
+
 
 class SlaveView(gobject.GObject):
     """
@@ -221,132 +193,130 @@ class SlaveView(gobject.GObject):
     """
     controller = None
     toplevel = None
-    widgets = None
+    widgets = []
     toplevel_name = None
     gladefile = None
     gladename = None
+    domain = None
 
     # This signal is emited when the view wants to return a result value
     gsignal("result", object)
-    
+
+    # This is emitted when validation changed for a view
+    # Used by parents views to know when child slaves changes
+    gsignal('validation-changed', bool)
+
     def __init__(self, toplevel=None, widgets=None, gladefile=None,
-                 gladename=None, toplevel_name=None):
+                 gladename=None, toplevel_name=None, domain=None):
         """ Creates a new SlaveView. Sets up self.toplevel and self.widgets
         and checks for reserved names.
         """
         gobject.GObject.__init__(self)
 
-        # setup the initial state with the value of the arguments or the
-        # class variables
+        self._broker = None
+        self.slaves = {}
+        self._proxies = []
+        self._valid = True
 
-        if not gladefile:
-            gladefile = self.gladefile
-        if not gladename:
-            gladename = self.gladename
-        if not toplevel_name:
-            toplevel_name = self.toplevel_name
-        if not toplevel:
-            toplevel = self.toplevel
-        self.gladefile = gladefile
-        self.gladename = gladename
-        self.toplevel_name = toplevel_name
-        self.toplevel = toplevel
-        self.widgets = widgets or self.widgets or []
-        
-        # stores the function that will be called when widgets 
+        # slave/widget name -> validation status
+        self._validation = {}
+
+        # stores the function that will be called when widgets
         # validity is checked
         self._validate_function = None
 
-        self.__broker = None
-        
+        # setup the initial state with the value of the arguments or the
+        # class variables
+        klass = type(self)
+        self.toplevel = toplevel or klass.toplevel
+        self.widgets = widgets or klass.widgets
+        self.gladefile = gladefile or klass.gladefile
+        self.gladename = gladename or klass.gladename
+        self.toplevel_name = (toplevel_name or
+                              klass.toplevel_name or
+                              self.gladefile or
+                              self.gladename)
+        self.domain = domain or klass.domain
+
+        self._check_reserved()
+        self._glade_adaptor = self.get_glade_adaptor()
+        self.toplevel = self._get_toplevel()
+
+        # grab the accel groups
+        self._accel_groups = gtk.accel_groups_from_object(self.toplevel)
+
+        # XXX: support normal widgets
+        # notebook page label widget ->
+        #   dict (slave name -> validation status)
+        self._notebook_validation = {}
+        self._notebooks = self._get_notebooks()
+
+    def _get_notebooks(self):
+        if not self._glade_adaptor:
+            return []
+
+        return [widget for widget in self._glade_adaptor.get_widgets()
+                           if isinstance(widget, gtk.Notebook)]
+
+    def _check_reserved(self):
         for reserved in ["widgets", "toplevel", "gladefile",
                          "gladename", "tree", "model", "controller"]:
             # XXX: take into account widget constructor?
             if reserved in self.widgets:
-                raise AttributeError("The widgets list for %s contains "
-                                     "a widget named `%s', which is "
-                                     "a reserved. name""" % (self, reserved))
+                raise AttributeError(
+                    "The widgets list for %s contains a widget named `%s', "
+                    "which is a reserved. name""" % (self, reserved))
 
+    def _get_toplevel(self):
+        toplevel = self.toplevel
+        if not toplevel and self.toplevel_name:
+            if self._glade_adaptor:
+                toplevel = self._glade_adaptor.get_widget(self.toplevel_name)
+            else:
+                toplevel = getattr(self, self.toplevel_name, None)
 
-        self.glade_adaptor = None
-        if self.gladefile is not None:
-            self._init_glade_adaptor()
-
-        if self.toplevel_name is not None and self.toplevel is None:
-            self.toplevel = getattr(self, self.toplevel_name, None)
-        
-        if not self.toplevel:
+        if not toplevel:
             raise TypeError("A View requires an instance variable "
                             "called toplevel that specifies the "
                             "toplevel widget in it")
 
+        if isinstance(toplevel, gtk.Window):
+            if toplevel.flags() & gtk.VISIBLE:
+                log.warn("Toplevel widget %s (%s) is visible; that's probably "
+                         "wrong" % (toplevel, toplevel.get_name()))
 
-        self.proxies = []
-        
-        # grab the accel groups
-        self._accel_groups = gtk.accel_groups_from_object(self.toplevel)
+        return toplevel
 
-        self.slaves = {}
-
-    # Global validations
-    def notify_validity(self):
-        """Checks if there are widgets with invalid data.
-        Calls the registered validate function
-        """
-        
-        if self._validate_function is None:
-            return
-        
-        valid = True
-        for widget_name in self.widgets:
-            widget = self.get_widget(widget_name)
-            if isinstance(widget, MixinSupportValidation):
-                if not widget.is_correct():
-                    valid = False
-                    break
-
-        self._validate_function(valid)
-
-    def register_validate_function(self, function):
-        """The signature of the validate function is:
-
-        def function(is_valid):
-
-        or, if it is a method:
-
-        def function(self, is_valid):
-
-        where the 'is_valid' parameter is True if all the widgets have
-        valid data or False otherwise.
-        """
-        self._validate_function = function
-
-    def _init_glade_adaptor(self):
+    def get_glade_adaptor(self):
         """Special init code that subclasses may want to override."""
-        self.glade_adaptor = WidgetTree(self, self.gladefile,
-                                        self.widgets, self.gladename)
+        if not self.gladefile:
+            return
 
-        container_name = self.toplevel_name or self.gladename or self.gladefile
-            
-        if container_name is None:
-            msg = ("You provided a gladefile %s to grab the widgets from "
-                   "but you didn't give me a toplevel/container name!")
-            raise ValueError(msg % self.gladefile)
+        glade_adaptor = _open_glade(self, self.gladefile,
+                                    self.widgets, self.gladename,
+                                    self.domain)
+
+        container_name = self.toplevel_name
+        if not container_name:
+            raise ValueError(
+                "You provided a gladefile %s to grab the widgets from "
+                "but you didn't give me a toplevel/container name!" %
+                self.gladefile)
 
         # a SlaveView inside a glade file needs to come inside a toplevel
         # window, so we pull our slave out from it, grab its groups and
         # muerder it later
-        shell = self.glade_adaptor.get_widget(container_name)
+        shell = glade_adaptor.get_widget(container_name)
         if not isinstance(shell, gtk.Window):
-            msg = "Container %s should be a Window, found %s"
-            raise TypeError(msg % (container_name, type(shell)))
-
-        # XXX grab the accel groups
+            raise TypeError("Container %s should be a Window, found %s" % (
+                container_name, type(shell)))
 
         self.toplevel = shell.get_child()
         shell.remove(self.toplevel)
         shell.destroy()
 
+        return glade_adaptor
+
     #
     # Hooks
     #
@@ -375,7 +345,7 @@ class SlaveView(gobject.GObject):
 
     def get_widget(self, name):
         """Retrieves the named widget from the View"""
-        name = string.replace(name,'.','_')
+        name = string.replace(name, '.', '_')
         widget = getattr(self, name, None)
         if widget is None:
             raise AttributeError("Widget %s not found in view %s"
@@ -399,13 +369,27 @@ class SlaveView(gobject.GObject):
     # GTK+ proxies and convenience functions
     #
 
+    def show_and_loop(self, parent=None):
+        """
+        Runs show() and runs the GTK+ event loop. If the parent
+        argument is supplied and is a valid view, this view is set as a
+        transient for the parent view
+
+        @param parent:
+        """
+
+        self.show()
+        if parent:
+            self.set_transient_for(parent)
+        gtk.main()
+
     def show(self, *args):
         """Shows the toplevel widget"""
         self.toplevel.show()
 
     def show_all(self, *args):
         """Shows all widgets attached to the toplevel widget"""
-        if self.glade_adaptor is not None:
+        if self._glade_adaptor is not None:
             raise AssertionError("You don't want to call show_all on a "
                                  "SlaveView. Use show() instead.")
         self.toplevel.show_all()
@@ -425,16 +409,16 @@ class SlaveView(gobject.GObject):
 
             - widgets: a list of widget names to be searched through
         """
-        widget = self.get_topmost_widget(widgets, can_focus=1)
-        if widget is not None: 
+        widget = self.get_topmost_widget(widgets, can_focus=True)
+        if widget is not None:
             widget.grab_focus()
         # So it can be idle_added safely
         return False
 
-    def get_topmost_widget(self, widgets=None, can_focus=0):
+    def get_topmost_widget(self, widgets=None, can_focus=False):
         """
         A real hack; returns the widget that is most to the left and
-        top of the window. 
+        top of the window.
 
             - widgets: a list of widget names.  If widgets is supplied,
               it only checks in the widgets in the list; otherwise, it
@@ -442,7 +426,8 @@ class SlaveView(gobject.GObject):
               self.widgets is None, looks through all widgets attached
               to the view.
 
-            - can_focus: boolean, if set only searches through widget that can focus
+            - can_focus: boolean, if set only searches through widget
+              that can be focused
         """
         # XXX: recurse through containers from toplevel widget, better
         # idea and will work.
@@ -460,8 +445,8 @@ class SlaveView(gobject.GObject):
                 if isinstance(widget.get_toplevel(), gtk.Window):
                     widget.realize()
                 else:
-                    _warn("get_topmost_widget: widget %s was unrealized"
-                          % widget_name)
+                    log.warn("get_topmost_widget: widget %s was not realized"
+                             % widget_name)
                     continue
             if can_focus:
                 # Combos don't focus, but their entries do
@@ -471,7 +456,7 @@ class SlaveView(gobject.GObject):
                     isinstance(widget, (gtk.Label, gtk.HSeparator,
                                         gtk.VSeparator, gtk.Window)):
                     continue
-                
+
             if top_widget:
                 allocation = widget.allocation
                 top_allocation = getattr(top_widget, 'allocation',  None)
@@ -488,22 +473,23 @@ class SlaveView(gobject.GObject):
     #
 
     def _attach_callbacks(self, controller):
-        if self.glade_adaptor is None:
+        if self._glade_adaptor is None:
             brokerclass = SignalBroker
         else:
             brokerclass = GladeSignalBroker
-            
-        self.__broker = brokerclass(self, controller)
+
+        self._broker = brokerclass(self, controller)
 
 #    def _setup_keypress_handler(self, keypress_handler):
 #        # Only useful in BaseView and derived classes
 #        # XXX: support slaveview correctly
-#        _warn("Tried to setup a keypress handler for %s "
+#        log.warn("Tried to setup a keypress handler for %s "
 #              "but no toplevel window exists to attach to" % self)
 
     #
     # Slave handling
     #
+
     def attach_slave(self, name, slave):
         """Attaches a slaveview to the current view, substituting the
         widget specified by name.  the widget specified *must* be a
@@ -511,10 +497,10 @@ class SlaveView(gobject.GObject):
         the specified slaveview's toplevel widget::
 
          .-----------------------. the widget that is indicated in the diagram
-         |window/view (self.view)| as placeholder will be substituted for the 
+         |window/view (self.view)| as placeholder will be substituted for the
          |  .----------------.   | slaveview's toplevel.
          |  | eventbox (name)|   |  .-----------------.
-         |  |.--------------.|      |slaveview (slave)| 
+         |  |.--------------.|      |slaveview (slave)|
          |  || placeholder  <----.  |.---------------.|
          |  |'--------------'|    \___ toplevel      ||
          |  '----------------'   |  ''---------------'|
@@ -524,11 +510,19 @@ class SlaveView(gobject.GObject):
         instead of the eventbox) is still supported for compatibility
         reasons but will print a warning.
         """
-        if not isinstance(slave, SlaveView):
-            _warn("slave specified must be a SlaveView, found %s" % slave)
+        log('%s: Attaching slave %s of type %s' % (self.__class__.__name__,
+                                                   name,
+                                                   slave.__class__.__name__))
 
-        if not hasattr(slave, "get_toplevel"):
-            raise TypeError("Slave does not have a get_toplevel method")
+        if name in self.slaves:
+            # XXX: TypeError
+            log.warn("A slave with name %s is already attached to %r"  % (
+                name, self))
+        self.slaves[name] = slave
+
+        if not isinstance(slave, SlaveView):
+            raise TypeError("slave must be a SlaveView, not a %s" %
+                            type(slave))
 
         shell = slave.get_toplevel()
 
@@ -540,11 +534,11 @@ class SlaveView(gobject.GObject):
 
         # if our widgets are in a glade file get the placeholder from them
         # or take it from the view itself otherwise
-        if self.glade_adaptor:
-            placeholder = self.glade_adaptor.get_widget(name)
+        if self._glade_adaptor:
+            placeholder = self._glade_adaptor.get_widget(name)
         else:
             placeholder = getattr(self, name, None)
-            
+
         if not placeholder:
             raise AttributeError(
                   "slave container widget `%s' not found" % name)
@@ -553,7 +547,8 @@ class SlaveView(gobject.GObject):
         if slave._accel_groups:
             # take care of accelerator groups; attach to parent window if we
             # have one; if embedding a slave into another slave, store its
-            # accel groups; otherwise complain if we're dropping the accelerators
+            # accel groups; otherwise complain if we're dropping the
+            # accelerators
             win = parent.get_toplevel()
             if isinstance(win, gtk.Window):
                 # use idle_add to be sure we attach the groups as late
@@ -563,19 +558,28 @@ class SlaveView(gobject.GObject):
             elif isinstance(self, SlaveView):
                 self._accel_groups.extend(slave._accel_groups)
             else:
-                _warn("attached slave %s to parent %s, but parent lacked "
-                      "a window and was not a slave view" % (slave, self))
+                log.warn("attached slave %s to parent %s, but parent lacked "
+                         "a window and was not a slave view" % (slave, self))
             slave._accel_groups = []
 
+        # Merge the sizegroups of the slave that is being attached  with the
+        # sizegroups of where it is being attached to. Only the sizegroups
+        # with the same name will be merged.
+        if slave._glade_adaptor:
+            sizegroups = slave._glade_adaptor.get_sizegroups()
+            for sizegroup in sizegroups:
+                self._merge_sizegroup(sizegroup)
+
         if isinstance(placeholder, gtk.EventBox):
             # standard mechanism
             child = placeholder.get_child()
             if child is not None:
                 placeholder.remove(child)
+            placeholder.set_visible_window(False)
             placeholder.add(new_widget)
         elif isinstance(parent, gtk.EventBox):
             # backwards compatibility
-            _warn("attach_slave's api has changed: read docs, update code!")
+            log.warn("attach_slave's api has changed: read docs, update code!")
             parent.remove(placeholder)
             parent.add(new_widget)
         else:
@@ -584,19 +588,55 @@ class SlaveView(gobject.GObject):
 
         # when attaching a slave we usually want it visible
         parent.show()
-        
         # call slave's callback
         slave.on_attach(self)
 
-        if self.slaves.has_key(name):
-            # I prefer to don't emit a raise here because it's really
-            # possible to change a certain slave in runtime.
-            _warn("You already have a slave %s attached" % name)
-        self.slaves[name] = slave
+        slave.connect('validation-changed',
+                      self._on_child__validation_changed,
+                      name)
+
+        for notebook in self._notebooks:
+            for child in notebook.get_children():
+                if not shell.is_ancestor(child):
+                    continue
+
+                label = notebook.get_tab_label(child)
+                slave.connect('validation-changed',
+                              self._on_notebook_slave__validation_changed,
+                              name, label)
+                self._notebook_validation[label] = {}
+
+        # Fire of an initial notification
+        slave.check_and_notify_validity(force=True)
 
         # return placeholder we just removed
         return placeholder
 
+    def _merge_sizegroup(self, other_sizegroup):
+        # Merge sizegroup from other with self that have the same name.
+        # Actually, no merging is being done, since the old group is preserved
+
+        name = other_sizegroup.get_data('gazpacho::object-id')
+        sizegroup = getattr(self, name, None)
+        if not sizegroup:
+            return
+
+        widgets = other_sizegroup.get_data('gazpacho::sizegroup-widgets')
+        if not widgets:
+            return
+
+        for widget in widgets:
+            sizegroup.add_widget(widget)
+
+    def detach_slave(self, name):
+        """
+        Detatch a slave called name from view
+        """
+        if not name in self.slaves:
+            raise LookupError("There is no slaved called %s attached to %r" %
+                              (name, self))
+        del self.slaves[name]
+
     def _attach_groups(self, win, accel_groups):
         # get groups currently attached to the window; we use them
         # to avoid reattaching an accelerator to the same window, which
@@ -617,23 +657,23 @@ class SlaveView(gobject.GObject):
             win.add_accel_group(group)
 
     def get_slave(self, holder):
-        if self.slaves.has_key(holder):
-            return self.slaves[holder]
-        raise AttributeError, "Unknown slave for holder: %s" % holder
+        return self.slaves.get(holder)
+
 
 
     #
     # Signal connection
     #
 
-    def connect_multiple(self, widgets, signal, handler, after=0):
+    def connect_multiple(self, widgets, signal, handler, after=False):
         """
         Connect the same handler to the specified signal for a number of
         widgets.
             - widgets: a list of GtkWidgets
             - signal: a string specifying the signals
             - handler: a callback method
-            - after: a boolean; if TRUE, we use connect_after(), otherwise, connect()
+            - after: a boolean; if TRUE, we use connect_after(), otherwise,
+            connect()
         """
         if not isinstance(widgets, (list, tuple)):
             raise TypeError("widgets must be a list, found %s" % widgets)
@@ -648,21 +688,21 @@ class SlaveView(gobject.GObject):
 
     def disconnect_autoconnected(self):
         """
-        Disconnect handlers previously connected with 
+        Disconnect handlers previously connected with
         autoconnect_signals()"""
-        self.__broker.disconnect_autoconnected()
-        
+        self._broker.disconnect_autoconnected()
+
     def handler_block(self, widget, signal_name=None):
         # XXX: Warning, or bail out?
-        if not self.__broker:
+        if not self._broker:
             return
-        self.__broker.handler_block(widget, signal_name)
+        self._broker.handler_block(widget, signal_name)
 
     def handler_unblock(self, widget, signal_name=None):
-        if not self.__broker:
+        if not self._broker:
             return
-        self.__broker.handler_unblock(widget, signal_name)
-        
+        self._broker.handler_unblock(widget, signal_name)
+
     #
     # Proxies
     #
@@ -671,64 +711,146 @@ class SlaveView(gobject.GObject):
         """
         Add a proxy to this view that automatically update a model when
         the view changes. Arguments:
-          
+
           - model. the object we are proxing. It can be None if we don't have
-          a model yet and we want to display the interface and set it up with
-          future models.
+            a model yet and we want to display the interface and set it up with
+            future models.
           - widgets. the list of widgets that contains model attributes to be
-          proxied. If it is None (or not specified) it will be the whole list
-          of widgets this View has.
+            proxied. If it is None (or not specified) it will be the whole list
+            of widgets this View has.
 
         This method return a Proxy object that you may want to use to force
         updates or setting new models. Keep a reference to it since there is
         no way to get that proxy later on. You have been warned (tm)
         """
+        log('%s: adding proxy for %s' % (
+            self.__class__.__name__,
+            model and model.__class__.__name__))
+
         widgets = widgets or self.widgets
+
+        for widget_name in widgets:
+            widget = getattr(self, widget_name, None)
+            if widget is None:
+                continue
+
+            if not IValidatableProxyWidget.providedBy(widget):
+                continue
+
+            try:
+                widget.connect('validation-changed',
+                               self._on_child__validation_changed,
+                               widget_name)
+            except TypeError:
+                raise AssertionError("%r does not have a validation-changed "
+                                     "signal." % widget)
+
         proxy = Proxy(self, model, widgets)
-        self.proxies.append(proxy)
+        self._proxies.append(proxy)
         return proxy
-    
-gobject.type_register(SlaveView)
+
+    #
+    # Validation
+    #
+
+    def _on_child__validation_changed(self, child, value, name):
+        # Children of the view, eg slaves or widgets are connected to
+        # this signal. When validation changes of a validatable child
+        # this callback is called
+        if isinstance(child, gtk.Widget):
+            # Force invisible and insensitive widgets to be valid
+            if (not child.get_property('visible') or
+                not child.get_property('sensitive')):
+                value = True
+
+        self._validation[name] = value
+
+        self.check_and_notify_validity()
+
+    def _on_notebook_slave__validation_changed(self, slave, value, name,
+                                               label):
+        validation = self._notebook_validation[label]
+        validation[name] = value
+
+        is_valid = True
+        if False in validation.values():
+            is_valid = False
+
+        if is_valid:
+            color = color_black
+        else:
+            color = color_red
+
+        # Only modify active state, since that's the (somewhat badly named)
+        # state used for the pages which are not selected.
+        label.modify_fg(gtk.STATE_ACTIVE, color)
+        label.modify_fg(gtk.STATE_NORMAL, color)
+
+    def check_and_notify_validity(self, force=False):
+        # Current view is only valid if we have no invalid children
+        # their status are stored as values in the dictionary
+        is_valid = True
+        if False in self._validation.values():
+            is_valid = False
+
+        # Check if validation really changed
+        if self._valid == is_valid and force == False:
+            return
+
+        self._valid = is_valid
+        self.emit('validation-changed', is_valid)
+
+        # FIXME: Remove and update all callsites to use validation-changed
+        if self._validate_function:
+            self._validate_function(is_valid)
+
+    def force_validation(self):
+        self.check_and_notify_validity(force=True)
+
+    def register_validate_function(self, function):
+        """The signature of the validate function is:
+
+        def function(is_valid):
+
+        or, if it is a method:
+
+        def function(self, is_valid):
+
+        where the 'is_valid' parameter is True if all the widgets have
+        valid data or False otherwise.
+        """
+        self._validate_function = function
+
+type_register(SlaveView)
 
 class BaseView(SlaveView):
     """A view with a toplevel window."""
-    
-    def __init__(self, toplevel=None, delete_handler=None, widgets=None,
-                 gladefile=None, gladename=None, toplevel_name=None):
-        """ toplevel is the widget to be set as `toplevel' (and which will
-        be aliased as `win'); delete_handler allows setting a function
-        to be called when this view's window is deleted."""
-        try:
-            SlaveView.__init__(self, toplevel, widgets, gladefile, gladename,
-                               toplevel_name)
-        except KeyError:
-            raise KeyError("Some widgets were defined in self.widgets "
-                           "but not found in the glade tree (see previous "
-                           "messages to see which ones).")
-            
+
+    def __init__(self, toplevel=None, widgets=None, gladefile=None,
+                 gladename=None, toplevel_name=None, domain=None,
+                 delete_handler=None):
+        SlaveView.__init__(self, toplevel, widgets, gladefile, gladename,
+                           toplevel_name, domain)
 
         if not isinstance(self.toplevel, gtk.Window):
             raise TypeError("toplevel widget must be a Window "
-                            "(or inherit from it),\nfound `%s' %s" 
+                            "(or inherit from it),\nfound `%s' %s"
                             % (toplevel, self.toplevel))
+        self.toplevel.set_name(self.__class__.__name__)
 
         if delete_handler:
-            id = self.toplevel.connect("delete_event", delete_handler)
+            id = self.toplevel.connect("delete-event", delete_handler)
             if not id:
                 raise ValueError(
                     "Invalid delete handler provided: %s" % delete_handler)
 
-    def _init_glade_adaptor(self):
-        self.glade_adaptor = WidgetTree(self, self.gladefile,
-                                        self.widgets, self.gladename)
-        name = self.toplevel_name or self.glade_adaptor.gladename
-        self.toplevel = self.glade_adaptor.get_widget(name)
+    def get_glade_adaptor(self):
+        if not self.gladefile:
+            return
+
+        return _open_glade(self, self.gladefile, self.widgets,
+                           self.gladename, self.domain)
 
-        if self.toplevel.flags() & gtk.VISIBLE:
-            _warn("Toplevel widget %s (%s) is visible; that's probably "
-                  "wrong" % (self.toplevel, name))
-            
-    
     #
     # Hook for keypress handling
     #
@@ -736,7 +858,7 @@ class BaseView(SlaveView):
     def _attach_callbacks(self, controller):
         super(BaseView, self)._attach_callbacks(controller)
         self._setup_keypress_handler(controller.on_key_press)
-        
+
     def _setup_keypress_handler(self, keypress_handler):
         self.toplevel.connect_after("key_press_event", keypress_handler)
 
@@ -788,8 +910,8 @@ class BaseView(SlaveView):
                 isinstance(v, _non_interactive)):
                 interactive = v
         if interactive:
-            _warn("No widget is focused in view %s but you have an "
-                  "interactive widget in it: %s""" % (self, interactive))
+            log.warn("No widget is focused in view %s but you have an "
+                     "interactive widget in it: %s""" % (self, interactive))
 
     #
     # Window show/hide and mainloop manipulation
@@ -816,7 +938,7 @@ class BaseView(SlaveView):
 
     def quit_if_last(self, *args):
         quit_if_last(*args)
-        
+
     def hide_and_quit(self, *args):
         """Hides the current window and breaks the GTK+ event loop if this
         is the last window.
@@ -824,3 +946,27 @@ class BaseView(SlaveView):
         """
         self.toplevel.hide()
         self.quit_if_last()
+
+WidgetTree = None
+
+def _open_glade(view, gladefile, widgets, name, domain):
+    global WidgetTree
+    if not WidgetTree:
+        try:
+            from gazpacho.loader import loader
+            loader # pyflakes
+        except ImportError, e:
+            if is_gazpacho_required():
+                raise RuntimeError(
+                    "Gazpacho is required, but could not be found: %s" % e)
+            else:
+                try:
+                    from kiwi.ui.libgladeloader import LibgladeWidgetTree as WT
+                    WidgetTree = WT
+                except ImportError:
+                    raise RuntimeError("Could not find a glade parser library")
+        else:
+            from kiwi.ui.gazpacholoader import GazpachoWidgetTree as WT
+            WidgetTree = WT
+
+    return WidgetTree(view, gladefile, widgets, name, domain)
