@@ -27,30 +27,19 @@
 
 import datetime
 
-import gtk
+import pango
 
-from kiwi.datatypes import ValidationError, converter, number
+from kiwi.datatypes import ValidationError, converter, number, \
+     ValueUnset
 from kiwi.decorators import deprecated
 from kiwi.python import deprecationwarn
 from kiwi.ui.entry import MaskError, KiwiEntry, ENTRY_MODE_TEXT, \
      ENTRY_MODE_DATA
 from kiwi.ui.dateentry import DateEntry
-from kiwi.ui.proxywidget import ValidatableProxyWidgetMixin
+from kiwi.ui.proxywidget import ValidatableProxyWidgetMixin, \
+     VALIDATION_ICON_WIDTH
 from kiwi.utils import PropertyObject, gsignal, type_register
 
-DATE_MASK_TABLE = {
-    '%m': '00',
-    '%y': '00',
-    '%d': '00',
-    '%Y': '0000',
-    '%H': '00',
-    '%M': '00',
-    '%S': '00',
-    '%T': '00:00:00',
-    # FIXME: locale specific
-    '%r': '00:00:00 LL',
-    }
-
 class ProxyEntry(KiwiEntry, ValidatableProxyWidgetMixin):
     """The Kiwi Entry widget has many special features that extend the basic
     gtk entry.
@@ -67,6 +56,9 @@ class ProxyEntry(KiwiEntry, ValidatableProxyWidgetMixin):
     how to fill these entries is displayed according to the current locale.
     """
 
+    allowed_data_types = (basestring, datetime.date, datetime.time,
+                          datetime.datetime, object) + number
+
     __gtype_name__ = 'ProxyEntry'
 
     def __init__(self, data_type=None):
@@ -78,22 +70,21 @@ class ProxyEntry(KiwiEntry, ValidatableProxyWidgetMixin):
     # Virtual methods
     gsignal('changed', 'override')
     def do_changed(self):
-        """Called when the content of the entry changes.
-
-        Sets an internal variable that stores the last time the user
-        changed the entry
-        """
-
-        self.chain()
-
+        if self._block_changed:
+            self.emit_stop_by_name('changed')
+            return
         self._update_current_object(self.get_text())
         self.emit('content-changed')
 
     def prop_set_data_type(self, data_type):
         data_type = super(ProxyEntry, self).prop_set_data_type(data_type)
+        # When there is no datatype, nothing needs to be done.
+        if not data_type:
+            return
 
-        # Numbers should be right aligned
-        if data_type and issubclass(data_type, number):
+        # Numbers and dates should be right aligned
+        if data_type and issubclass(data_type, (number, datetime.date,
+                                    datetime.time, datetime.datetime)):
             self.set_property('xalign', 1.0)
 
         # Apply a mask for the data types, some types like
@@ -110,24 +101,8 @@ class ProxyEntry(KiwiEntry, ValidatableProxyWidgetMixin):
         """
         @param data_type:
         """
-
-        if not data_type in (datetime.datetime, datetime.date, datetime.time):
-            return
         conv = converter.get_converter(data_type)
-        mask = conv.get_format()
-
-        # For win32, skip mask
-        # FIXME: How can we figure out the real locale specific string?
-        if mask == '%X':
-            mask = '%H:%M:%S'
-        elif mask == '%x':
-            mask = '%d/%m/%Y'
-        elif mask == '%c':
-            mask = '%d/%m/%Y %H:%M:%S'
-
-        for format_char, mask_char in DATE_MASK_TABLE.items():
-            mask = mask.replace(format_char, mask_char)
-
+        mask = conv.get_mask()
         self.set_mask(mask)
 
     #@deprecated('prefill')
@@ -168,18 +143,12 @@ class ProxyEntry(KiwiEntry, ValidatableProxyWidgetMixin):
         # Protect content-changed from being updated and issue
         # a manual emission afterwards
         self._block_changed = True
-        gtk.Entry.set_text(self, text)
+        KiwiEntry.set_text(self, text)
         self._block_changed = False
         self.emit('content-changed')
 
         self.set_position(-1)
 
-    def do_changed(self):
-        if self._block_changed:
-            self.emit_stop_by_name('changed')
-            return
-        self.emit('content-changed')
-
     # ProxyWidgetMixin implementation
 
     def read(self):
@@ -190,9 +159,9 @@ class ProxyEntry(KiwiEntry, ValidatableProxyWidgetMixin):
                 return self._from_string(text)
             except ValidationError:
                 # Do not consider masks which only displays static
-                # characters invalid, instead return an empty string
+                # characters invalid, instead return None
                 if self.get_mask() and text == self.get_empty_mask():
-                    return ""
+                    return
                 else:
                     raise
         elif mode == ENTRY_MODE_DATA:
@@ -201,7 +170,7 @@ class ProxyEntry(KiwiEntry, ValidatableProxyWidgetMixin):
             raise AssertionError
 
     def update(self, data):
-        if data is None:
+        if data is None or data is ValueUnset:
             text = ""
         else:
             mode = self._mode
@@ -236,6 +205,23 @@ class ProxyDateEntry(PropertyObject, DateEntry, ValidatableProxyWidgetMixin):
         ValidatableProxyWidgetMixin.__init__(self)
         PropertyObject.__init__(self)
 
+        # Add some space to the entry, so it has rom for the icon, in case
+        # of a validation error.
+        #
+        # Since we set the widget's width based on the number of characters,
+        # get the width of a single char, so we can calculate how many
+        # 'chars' the icon takes.
+        layout = self.entry.get_layout()
+        context = layout.get_context()
+        metrics = context.get_metrics(context.get_font_description())
+        char_width =  metrics.get_approximate_char_width() / pango.SCALE
+        current_width = self.entry.get_width_chars()
+    
+        # We add 4 pixels to the width, because of the icon borders
+        icon_width = VALIDATION_ICON_WIDTH + 4
+        self.entry.set_width_chars(current_width
+                                   + int(icon_width / char_width))
+
     gsignal('changed', 'override')
     def do_changed(self):
         self.chain()
@@ -252,4 +238,27 @@ class ProxyDateEntry(PropertyObject, DateEntry, ValidatableProxyWidgetMixin):
         else:
             self.set_date(data)
 
+    def prop_set_mandatory(self, value):
+        self.entry.set_property('mandatory', value)
+        return value
+
+    # ValidatableProxyWidgetMixin
+
+    # Since the widget that should be marked as valid/invalid is the entry,
+    # we also call those methods for self.entry
+    def set_valid(self):
+        ValidatableProxyWidgetMixin.set_valid(self)
+        self.entry.set_valid()
+
+    def set_invalid(self, text=None, fade=True):
+        ValidatableProxyWidgetMixin.set_invalid(self, text, fade)
+        self.entry.set_invalid(text, fade)
+
+    def set_blank(self):
+        ValidatableProxyWidgetMixin.set_blank(self)
+        self.entry.set_blank()
+
+    def get_background(self):
+        return self.entry.get_background()
+
 type_register(ProxyDateEntry)
